using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using System.Collections.Generic;
using CreditAgricole.Framework;

namespace CreditAgricole.CommonElements.Features.CA_Entities_List
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("bf398ae6-479f-4db6-b0fe-16e4d35b955a")]
    public class CA_Entities_ListEventReceiver : SPFeatureReceiver
    {
        // Uncomment the method below to handle the event raised after a feature has been activated.

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            CALoggingHelper.AddInformationEntry(string.Format("Feature {0} ({1}) activated.", properties.Feature.Definition.DisplayName, properties.Feature.DefinitionId));
            SPHelper.EnforceUniqueValue(properties.Feature.Parent as SPWeb, Constants.EntitiesListTitle, SPBuiltInFieldId.Title);
        }


        // Uncomment the method below to handle the event raised before a feature is deactivated.

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            CALoggingHelper.AddInformationEntry(string.Format("Feature {0} ({1}) deactivated.", properties.Feature.Definition.DisplayName, properties.Feature.DefinitionId));
        }


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
            CALoggingHelper.AddInformationEntry(string.Format("Feature {0} ({1}) uninstalled.", properties.Feature.Definition.DisplayName, properties.Feature.DefinitionId));
        }

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
