using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using System.IO;
using CreditAgricole.Framework;
using Microsoft.SharePoint.Utilities;

namespace CreditAgricole.PageLayout.Features.CreditAgricoleMasterPageFeature
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("22e461b3-f787-43e0-ad01-0bc9e9be1b12")]
    public class CreditAgricoleMasterPageFeatureEventReceiver : SPFeatureReceiver
    {
        // Uncomment the method below to handle the event raised after a feature has been activated.

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPSite site = properties.Feature.Parent as SPSite;
            SPWeb rootWeb = site.RootWeb;
            string directory = properties.Feature.Definition.RootDirectory;
            directory += @"\ITMasterModule";
            try
            {
                string masterpagePath = string.Empty;
                string serverRelativeUrl = SPContext.Current.Site.ServerRelativeUrl;
                if (serverRelativeUrl == "/")
                {
                    masterpagePath = string.Format("/_catalogs/masterpage/{0}", Constants.MasterpagetName);
                }
                else
                {
                    masterpagePath = string.Format("{0}/_catalogs/masterpage/{1}", serverRelativeUrl, Constants.MasterpagetName);
                }
                SPFile homePageLayoutFile = rootWeb.GetFile(masterpagePath);
                if (homePageLayoutFile.Exists)
                {
                    string listName = SPUtility.GetLocalizedString("$Resources:core,Master Page Gallery", "core", rootWeb.Language);
                    SPList masterPageList = rootWeb.Lists.TryGetList(listName);
                    if (masterpagePath != null)
                    {
                        SPFolder masterpagesFolder = rootWeb.Lists.TryGetList(listName).RootFolder;
                        UpdateMasterPage(homePageLayoutFile, masterpagesFolder, directory, Constants.MasterpagetName);
                    }
                }
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
            catch (ArgumentException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
        }

        private void UpdateMasterPage(SPFile oFile, SPFolder masterpagesFolder, string directory, string oPropertyName)
        {
            try
            {
                if (oFile.CheckOutStatus != SPFile.SPCheckOutStatus.None)
                {
                    oFile.CheckIn("auto checkin to update pae layout");
                }
                oFile.CheckOut();
                string[] templates = Directory.GetFiles(directory, oPropertyName, System.IO.SearchOption.TopDirectoryOnly);
                string pageLayoutFileName = templates[0];
                FileInfo fileInfo = new FileInfo(pageLayoutFileName);
                byte[] byteArr = System.IO.File.ReadAllBytes(pageLayoutFileName);
                masterpagesFolder.Files.Add(fileInfo.Name, byteArr, true);
                oFile.Update();
                oFile.CheckIn("Check in");
                oFile.Publish("Published");
                oFile.Approve("Approved");
            }
            catch (SPException ex)
            {
                CALoggingHelper.AddErrorEntry(ex.StackTrace);
            }
        }

        // Uncomment the method below to handle the event raised before a feature is deactivated.

        //public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
